package operator.selection;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;

import core.Operator;
import core.Solution;
import utils.PseudoRandomGenerator;
import utils.comparator.BinaryTournamentComparator;

public class BinaryTournamentSelection extends Operator{

	private Comparator comparator_;

	public BinaryTournamentSelection(HashMap<String, Object> parameters) {
		super(parameters);
		if ((parameters != null) && (parameters.get("comparator") != null)) {
			this.comparator_ = (Comparator)parameters.get("comparator");
		} else if((parameters != null) && parameters.get("secondComparator") != null){
			this.comparator_ = new BinaryTournamentComparator((Comparator)parameters.get("secondComparator"));
		} else {
			this.comparator_ = new BinaryTournamentComparator();
		}
	}


	public Comparator getComparator() {
		return this.comparator_;
	}

	@Override
	public Object execute(Object object) throws ClassNotFoundException {
		ArrayList<Solution> solutionSet = (ArrayList<Solution>)object;
		Solution s1, s2;
		s1 = new Solution(solutionSet.get(PseudoRandomGenerator.randInt(0, solutionSet.size())));
		s2 = new Solution(solutionSet.get(PseudoRandomGenerator.randInt(0, solutionSet.size())));

		int flag = comparator_.compare(s1, s2);

		if(flag == -1) {
			return s1;
		} else if(flag == 1) {
			return s2;
		} else if(PseudoRandomGenerator.randDouble() < 0.5) {
			return s1;
		} else {
			return s2;
		}
	}

}
